/********************************************************************
  *
  *  program     : KundenBroker.java
  *  author      : heljanic
  *  version     : 2001-04-24 (001)
  *  (c)         : Helge Janicke, Niels-Peter de Witt, Karsten Wolke
  *  email       : heljanic@agent.fho-emden.de
  *		 		 : ndewitt@gmx.de
  *		 		 : mail@karsten-wolke.de
  *  description : Broker for class Kunde.
  *
  *******************************************************************
  * modifications: See at the end of the document.
  *******************************************************************
  * comments:
  *******************************************************************
  * license:
  *	This software is is provided AS IS, without a warranty of any
  *	kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATION AND
  *	WARRANTIES, INCLUDING ANY IMLIED WARRANTY OF MERCHANTABILITY,
  *	FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
  *	HEREBY EXCLUDED. WE SHALL NOT BE LIABLE FOR ANY DAMAGES
  *	SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
  *	DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WE
  *	WILL BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR
  *	DIRECT, INDIRECT, SPECIAL, CENSEQUENTIAL, INCIDENTIAL OR
  *	PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY
  *	OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
  *	SOFTWARE, EVEN IF WE HAVE BEEN ADVISED OF THE POSSIBILITY OF
  *	SUCH DAMAGES.
  *	This software is not designed or intended for use in online
  *	control of aircraft, air traffic, aircraft navigation or
  *	aircraft communications; or in the design, construction,
  *	operation or maintenance of any nuclear facility. Licensee
  *	represents and warrants that it will not use or redistribute
  *	the software for such purposes.
********************************************************************/
import de.web.databasesupport.tools.database.*;
import java.sql.*;

/** The KundenBroker class is responsible for reading and writing Kunde objects
  * from and to the database. This class holds a reference to a Database object
  * which represents the database connection. The connection have to be established
  * when instanciating this class, because it uses PreparedStatements.
  * A NotConnectedException will be thrown if the connection is not established.
  */
public class KundenBroker {

	/** String containing the SQL statement used to read a Kunde object. */
	private final static String strKundeByKnr = "SELECT kunde.nr, kunde.name, kunde.strasse, kunde.ort FROM kunde WHERE kunde.nr = ? ; ";

	/** String containing the SQL statement used to write a Kunde object. */
  	private final static String strUpdateKundeByKnr = "UPDATE kunde SET kunde.name = ? , kunde.strasse = ? , kunde.ort = ? WHERE knr = ? ; ";

	/** The Database object representing the connection. */
	private Database db = null;

	/** The PreparedStatement used to read a Kunde object. */
	private PreparedStatement preKundeByKnr = null;

	/** The PreparedStatement used to write a Kunde object. */
	private PreparedStatement preUpdateKundeByKnr = null;

	/** Constructor.
	  * Ensure that the Database connection is established when instanciating this class.
	  * @param db The Database object representing the connection.
	  */
	public KundenBroker(Database db) throws SQLException, NotConnectedException {
		this.db = db;
		this.preKundeByKnr = db.precompile(strKundeByKnr);
		this.preUpdateKundeByKnr = db.precompile(strUpdateKundeByKnr);
	}

	/** Writes a Kunde object to the database.
	  * A Kunde object is identified by the Kunde.knr property. The name, strasse, ort
	  *	property is updated.
	  * @param kunde The Kunde object to write to the database.
	  * @throw SQLException If there is no Kunde with the specified knr
	  *		or the used SQLStatement (strUpdateKundeByKnr) was wrong.
	  * @throw NotConnectedException If the Database connetion is not established.
	  */
	public void writeKunde(Kunde kunde) throws SQLException, NotConnectedException {
		preUpdateKundeByKnr.clearParameters();
		preUpdateKundeByKnr.setString(1, kunde.getName());
		preUpdateKundeByKnr.setString(2, kunde.getStrasse());
		preUpdateKundeByKnr.setString(3, kunde.getOrt());
		preUpdateKundeByKnr.setInt(4, kunde.getKnr());
		preUpdateKundeByKnr.executeUpdate();
		db.commit();
	}

	/** Reads a Kunde object from the database.
	  * @param knr The Knr property of the Kunde to retrieve.
	  * @return The Kunde. null if no Kunde was found.
	  * @throw SQLException if the used SQLStatement (strKundeByKnr) was wrong.
	  */
	public Kunde readKunde(int knr) throws SQLException {
		Kunde returnKunde = null;
		preKundeByKnr.clearParameters();
		preKundeByKnr.setInt(1, knr);
		ResultSet result = preKundeByKnr.executeQuery();
		if (result.next()) {
			returnKunde = new Kunde();
			returnKunde.setKnr(result.getInt(1));
			returnKunde.setName(result.getString(2));
			returnKunde.setStrasse(result.getString(3));
			returnKunde.setOrt(result.getString(4));
		}
		result.close();
		return returnKunde;
	}

}

/********************************************************************
  * date: ******* who: ****** description: **************************
  * -----------------------------------------------------------------
  * 2000-10-19    heljanic    your description here
  * -----------------------------------------------------------------
********************************************************************/
